@@ -15,15 +15,17 @@
  */
 
 
-#include <bson.h>
+#include <bson/bson.h>
 
-#include "mongoc-array-private.h"
+#include "bson/bson-types.h"
+
+#include "mongoc/mongoc-array-private.h"
 /* For strcasecmp on Windows */
-#include "mongoc-util-private.h"
-#include "mongoc-write-concern.h"
-#include "mongoc-write-concern-private.h"
-#include "mongoc-cluster-private.h"
-#include "mongoc-client-private.h"
+#include "mongoc/mongoc-util-private.h"
+#include "mongoc/mongoc-write-concern.h"
+#include "mongoc/mongoc-write-concern-private.h"
+#include "mongoc/mongoc-cluster-private.h"
+#include "mongoc/mongoc-client-private.h"
 
 #include "test-conveniences.h"
 #include "TestSuite.h"
@@ -175,7 +177,7 @@ bson_lookup_value (const bson_t *b, const char *key, bson_value_t *value)
    bson_iter_t iter;
    bson_iter_t descendent;
 
-   bson_iter_init (&iter, b);
+   BSON_ASSERT (bson_iter_init (&iter, b));
    BSON_ASSERT (bson_iter_find_descendant (&iter, key, &descendent));
    bson_value_copy (bson_iter_value (&descendent), value);
 }
@@ -208,7 +210,7 @@ bson_lookup_doc_null_ok (const bson_t *b, const char *key, bson_t *doc)
    bson_iter_t iter;
    bson_iter_t descendent;
 
-   bson_iter_init (&iter, b);
+   BSON_ASSERT (bson_iter_init (&iter, b));
    BSON_ASSERT (bson_iter_find_descendant (&iter, key, &descendent));
    if (!BSON_ITER_HOLDS_NULL (&descendent)) {
       bson_iter_bson (&descendent, doc);
@@ -302,7 +304,7 @@ bson_lookup_write_concern (const bson_t *b, const char *key)
    bson_iter_t w;
 
    bson_lookup_doc (b, key, &doc);
-   bson_iter_init (&iter, &doc);
+   BSON_ASSERT (bson_iter_init (&iter, &doc));
 
    /* current command monitoring tests always have "w" and no other fields */
    ASSERT_CMPUINT32 (bson_count_keys (&doc), ==, (uint32_t) 1);
@@ -509,7 +511,8 @@ find (bson_value_t *value,
       const bson_t *doc,
       const char *key,
       bool is_command,
-      bool is_first);
+      bool is_first,
+      bool retain_dots_in_keys);
 
 
 /*--------------------------------------------------------------------------
@@ -690,6 +693,8 @@ derive (match_ctx_t *ctx, match_ctx_t *derived, const char *key)
    } else {
       bson_snprintf (derived->path, sizeof derived->path, "%s", key);
    }
+   derived->retain_dots_in_keys = ctx->retain_dots_in_keys;
+   derived->allow_placeholders = ctx->allow_placeholders;
 }
 
 
@@ -755,7 +760,8 @@ match_bson_with_ctx (const bson_t *doc,
    while (bson_iter_next (&pattern_iter)) {
       key = bson_iter_key (&pattern_iter);
       value = bson_iter_value (&pattern_iter);
-      found = find (&doc_value, doc, key, is_command, is_first);
+      found = find (
+         &doc_value, doc, key, is_command, is_first, ctx->retain_dots_in_keys);
 
       /* is value {"$exists": true} or {"$exists": false} ? */
       is_exists_operator = get_exists_operator (value, &exists);
@@ -819,14 +825,15 @@ find (bson_value_t *value,
       const bson_t *doc,
       const char *key,
       bool is_command,
-      bool is_first)
+      bool is_first,
+      bool retain_dots_in_keys)
 {
    bson_iter_t iter;
    bson_iter_t descendent;
 
    bson_iter_init (&iter, doc);
 
-   if (strchr (key, '.')) {
+   if (!retain_dots_in_keys && strchr (key, '.')) {
       if (!bson_iter_find_descendant (&iter, key, &descendent)) {
          return false;
       }
@@ -1034,6 +1041,22 @@ match_bson_value (const bson_value_t *doc,
    int64_t pattern_int64;
    bool ret;
 
+   if (ctx && ctx->allow_placeholders) {
+      /* The change streams spec tests use the value 42 as a placeholder. */
+      bool is_placeholder = false;
+      if (is_number_type (pattern->value_type) &&
+          bson_value_as_int64 (pattern) == 42) {
+         is_placeholder = true;
+      }
+      if (pattern->value_type == BSON_TYPE_UTF8 &&
+          !strcmp (pattern->value.v_utf8.str, "42")) {
+         is_placeholder = true;
+      }
+      if (is_placeholder) {
+         return true;
+      }
+   }
+
    if (is_number_type (pattern->value_type) && ctx &&
        !ctx->strict_numeric_types) {
       doc_int64 = bson_value_as_int64 (doc);
@@ -1361,3 +1384,157 @@ assert_no_duplicate_keys (const bson_t *doc)
 
    mongoc_set_destroy (keys);
 }
+
+
+void
+match_in_array (const bson_t *doc, const bson_t *array, match_ctx_t *ctx)
+{
+   bson_iter_t array_iter;
+   bool found = false;
+
+   BSON_ASSERT (bson_iter_init (&array_iter, array));
+
+   while (bson_iter_next (&array_iter)) {
+      bson_t array_elem;
+
+      ASSERT (BSON_ITER_HOLDS_DOCUMENT (&array_iter));
+      bson_iter_bson (&array_iter, &array_elem);
+
+      if (match_bson_with_ctx (&array_elem, doc, false, ctx)) {
+         found = true;
+      }
+
+      bson_destroy (&array_elem);
+   }
+   if (!found) {
+      test_error ("could not match: %s\n\n"
+                  "in array:\n%s\n\n",
+                  bson_as_canonical_extended_json (doc, NULL),
+                  bson_as_canonical_extended_json (array, NULL));
+   }
+}
+
+bson_t *
+bson_with_all_types ()
+{
+   bson_t *bson = tmp_bson ("{}");
+   bson_oid_t oid;
+   bson_decimal128_t dec;
+
+   BSON_ASSERT (bson_decimal128_from_string ("1.23456789", &dec));
+   bson_oid_init_from_string (&oid, "000000000000000000000000");
+   BSON_ASSERT (BSON_APPEND_DOUBLE (bson, "double", 1.0));
+   BSON_ASSERT (BSON_APPEND_UTF8 (bson, "string", "string_example"));
+   BSON_ASSERT (
+      BSON_APPEND_DOCUMENT (bson, "document", tmp_bson ("{'x': 'y'}")));
+   BSON_ASSERT (BSON_APPEND_ARRAY (bson, "document", tmp_bson ("{'0': 'x'}")));
+   BSON_ASSERT (BSON_APPEND_BINARY (
+      bson, "binary", BSON_SUBTYPE_BINARY, (uint8_t *) "data", 4));
+   BSON_ASSERT (BSON_APPEND_UNDEFINED (bson, "undefined"));
+   BSON_ASSERT (BSON_APPEND_OID (bson, "oid", &oid));
+   BSON_ASSERT (BSON_APPEND_BOOL (bson, "bool", true));
+   BSON_ASSERT (BSON_APPEND_DATE_TIME (bson, "datetime", 123));
+   BSON_ASSERT (BSON_APPEND_NULL (bson, "null"));
+   BSON_ASSERT (BSON_APPEND_REGEX (bson, "regex", "a+", NULL));
+   BSON_ASSERT (BSON_APPEND_DBPOINTER (bson, "dbpointer", "collection", &oid));
+   BSON_ASSERT (BSON_APPEND_CODE (bson, "code", "var x = 1;"));
+   BSON_ASSERT (BSON_APPEND_SYMBOL (bson, "symbol", "symbol_example"));
+   BSON_ASSERT (BSON_APPEND_CODE (bson, "code", "var x = 1;"));
+   BSON_ASSERT (BSON_APPEND_CODE_WITH_SCOPE (
+      bson, "code_w_scope", "var x = 1;", tmp_bson ("{}")));
+   BSON_ASSERT (BSON_APPEND_INT32 (bson, "int32", 1));
+   BSON_ASSERT (BSON_APPEND_TIMESTAMP (bson, "timestamp", 2, 3));
+   BSON_ASSERT (BSON_APPEND_INT64 (bson, "int64", 4));
+   BSON_ASSERT (BSON_APPEND_DECIMAL128 (bson, "decimal128", &dec));
+   BSON_ASSERT (BSON_APPEND_MINKEY (bson, "minkey"));
+   BSON_ASSERT (BSON_APPEND_MAXKEY (bson, "maxkey"));
+   /* and an empty key, as it so often is an edge case. */
+   BSON_ASSERT (BSON_APPEND_INT32 (bson, "", -1));
+   return bson;
+}
+
+const char *
+json_with_all_types ()
+{
+   const char *json = "{\n"
+                      "    \"double\": {\n"
+                      "        \"$numberDouble\": \"1.0\"\n"
+                      "    },\n"
+                      "    \"string\": \"string_example\",\n"
+                      "    \"document\": {\n"
+                      "        \"x\": \"y\"\n"
+                      "    },\n"
+                      "    \"document\": [\"x\"],\n"
+                      "    \"binary\": {\n"
+                      "        \"$binary\": {\n"
+                      "            \"base64\": \"ZGF0YQ==\",\n"
+                      "            \"subType\": \"00\"\n"
+                      "        }\n"
+                      "    },\n"
+                      "    \"undefined\": {\n"
+                      "        \"$undefined\": true\n"
+                      "    },\n"
+                      "    \"oid\": {\n"
+                      "        \"$oid\": \"000000000000000000000000\"\n"
+                      "    },\n"
+                      "    \"bool\": true,\n"
+                      "    \"datetime\": {\n"
+                      "        \"$date\": {\n"
+                      "            \"$numberLong\": \"123\"\n"
+                      "        }\n"
+                      "    },\n"
+                      "    \"null\": null,\n"
+                      "    \"regex\": {\n"
+                      "        \"$regularExpression\": {\n"
+                      "            \"pattern\": \"a+\",\n"
+                      "            \"options\": \"\"\n"
+                      "        }\n"
+                      "    },\n"
+                      "    \"dbpointer\": {\n"
+                      "        \"$dbPointer\": {\n"
+                      "            \"$ref\": \"collection\",\n"
+                      "            \"$id\": {\n"
+                      "                \"$oid\": \"000000000000000000000000\"\n"
+                      "            }\n"
+                      "        }\n"
+                      "    },\n"
+                      "    \"code\": {\n"
+                      "        \"$code\": \"var x = 1;\"\n"
+                      "    },\n"
+                      "    \"symbol\": {\n"
+                      "        \"$symbol\": \"symbol_example\"\n"
+                      "    },\n"
+                      "    \"code\": {\n"
+                      "        \"$code\": \"var x = 1;\"\n"
+                      "    },\n"
+                      "    \"code_w_scope\": {\n"
+                      "        \"$code\": \"var x = 1;\",\n"
+                      "        \"$scope\": {}\n"
+                      "    },\n"
+                      "    \"int32\": {\n"
+                      "        \"$numberInt\": \"1\"\n"
+                      "    },\n"
+                      "    \"timestamp\": {\n"
+                      "        \"$timestamp\": {\n"
+                      "            \"t\": 2,\n"
+                      "            \"i\": 3\n"
+                      "        }\n"
+                      "    },\n"
+                      "    \"int64\": {\n"
+                      "        \"$numberLong\": \"4\"\n"
+                      "    },\n"
+                      "    \"decimal128\": {\n"
+                      "        \"$numberDecimal\": \"1.23456789\"\n"
+                      "    },\n"
+                      "    \"minkey\": {\n"
+                      "        \"$minKey\": 1\n"
+                      "    },\n"
+                      "    \"maxkey\": {\n"
+                      "        \"$maxKey\": 1\n"
+                      "    },\n"
+                      "    \"\": {\n"
+                      "        \"$numberInt\": \"-1\"\n"
+                      "    }\n"
+                      "}";
+   return json;
+}
\ No newline at end of file
